<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\Novocodigo as NovocodigoRepository;
use ACL\Traits\TCrud;

class Novocodigo {

    use TCrud;

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;

    /**
     * @var NovocodigoRepository
     */
    private $novocodigoRepository;

    public function __construct(NovocodigoRepository $novocodigoRepository)
    {
        $this->novocodigoRepository = $novocodigoRepository;
        $this->libEnvironment = InstanceProvider::getLibEnvironment();
    }

    /**
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Doctrine\DBAL\Exception
     */
    public function validateNovocodigo()
    {
        $entityManager = InstanceProvider::getEntityManager();
        $organizationId = $this->libEnvironment->getOrganizationId();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $operator = $this->novocodigoRepository->validateNovocodigo($organizationId, "OPERADOR", "CDOPERADOR")[0];
            if($operator["VALIDACAO"] == 1){
                $this->updateNovocodigo($operator["ID"], 'OPERADOR');
            }
            $perfilacesso = $this->novocodigoRepository->validateNovocodigo($organizationId, "PERFILACESSO", "NRPERFILACESSO")[0];
            if($perfilacesso["VALIDACAO"] == 1){
                $this->updateNovocodigo($perfilacesso["ID"], 'PERFILACESSO');
            }
            $perfilacessoparc = $this->novocodigoRepository->validateNovocodigo($organizationId, "PERFACESSPARC", "NRPERFACESSPARC")[0];
            if($perfilacessoparc["VALIDACAO"] == 1){
                $this->updateNovocodigo($perfilacessoparc["ID"], 'PERFACESSPARC');
            }
            $parceria = $this->novocodigoRepository->validateNovocodigo($organizationId, "PARCERIA", "NRPARCERIA")[0];
            if($parceria["VALIDACAO"] == 1){
                $this->updateNovocodigo($parceria["ID"], 'PARCERIA');
            }
            $organizacaoOper = $this->novocodigoRepository->validateNovocodigo($organizationId, "ORGANIZACAOOPER", "NRORGANIZACAOOPER")[0];
            if($organizacaoOper["VALIDACAO"] == 1){
                $this->updateNovocodigo($organizacaoOper["ID"], 'ORGANIZACAOOPER');
            }
            $parcNegocio = $this->novocodigoRepository->validateNovocodigo($organizationId, "PARCNEGOCIO", "NRPARCNEGOCIO")[0];
            if($organizacaoOper["VALIDACAO"] == 1){
                $this->updateNovocodigo($parcNegocio["ID"], 'PARCNEGOCIO');
            }

            try {
                $relacParcNeg = $this->novocodigoRepository->validateNovocodigo($organizationId, "RELACPARCNEG", "NRRELACPARCNEG")[0];
                if($organizacaoOper["VALIDACAO"] == 1){
                    $this->updateNovocodigo($organizacaoOper["ID"], 'RELACPARCNEG');
                }
            } catch (\Exception $e) {

            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function updateNovocodigo($newCode, $table)
    {
        $newCode = strval(str_pad($newCode, 20, "0", STR_PAD_LEFT));
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $this->novocodigoRepository->updateNovocodigo($table, $newCode, $organizationId);
    }
}
