import { Http, Loader, Metadata, Singleton } from '@zeedhi/core';
import { OperadorService } from '@/services/OperadorService';
import { Form, IGridEvent, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Date as DateInput } from '@zeedhi/common/types/components/zd-date/date';
import { AclAlertService } from '@/services/AclAlertService';
import AclMessages from '@/Util/AclMessages';
import { AclDialogService } from '@/services/AclDialogService';
import { AclHttp } from '@/services/AclHttp';

@Singleton
export class OperadorPerfilDeAcessoTabController {
	public operadorService!: OperadorService;

	constructor() {
		this.operadorService = new OperadorService();
	}

	public operadorController = Loader.getInstance('OperadorController');

	public getCurrentNrparcnegocio(): string {
		return this.operadorController.currentOperador.NRPARCNEGOCIO;
	}

	public async clickTab() {
		this.operadorService.clickTabOperadorPerfilAcesso(this.operadorController.currentOperador);
	}

	private getInstanceOperadorPerfilDeAcessoGrid() {
		return Metadata.getInstance<TekGrid>('operadorPerfilDeAcessoGrid');
	}

	private getInstanceAddPerfilByOperadorGrid() {
		return Metadata.getInstance<TekGrid>('addPerfilByOperadorGrid');
	}

	public async addClickOperadorPerfilDeAcessoGrid() {
		await this.createAddPerfilByOperadorModal();
		await this.addPerfilAcessoByOperadorModal.show();
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		const nrparcnegocio = this.getCurrentNrparcnegocio();
		await addGrid.datasource.addFilter('NRPARCNEGOCIO', nrparcnegocio);
	}

	public async beforeDeleteOperadorPerfilDeAcessoGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await AclDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deletePerfilacesso();
		}
	}

	public async deletePerfilacesso() {
		const grid = this.getInstanceOperadorPerfilDeAcessoGrid();
		await AclHttp.$delete('/perfacessparc', grid.selectedRows);
		AclAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
		await grid.reload();
	}

	/**
	 * Modal add perfil by operador
	 */
	public addPerfilAcessoByOperadorModal!: Modal;

	public async createAddPerfilByOperadorModal() {
		this.addPerfilAcessoByOperadorModal = await ModalService.createFromJson('components/operador/modais/addPerfilByOperador', true);
	}

	public async closeAddPerfilByOperadorModal() {
		this.addPerfilAcessoByOperadorModal.hide();
		this.addPerfilAcessoByOperadorModal?.destroy();
		await this.getInstanceOperadorPerfilDeAcessoGrid().reload();
	}

	public get validateSaveButtonAddPerfilByOperadorModal(): boolean {
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		return !addGrid.selectedRows.length;
	}

	public async saveAddOperadorByPerfilModal() {
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		const addPerfilModalForm = Metadata.getInstance<Form>('addPerfilByOperadorForm');
		const dtValidateField = addPerfilModalForm.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = addGrid;
		selectedRows.forEach((row) => {
			row.DTVALIDADE = dtValidateField.value;
			row.NRPARCNEGOCIO = this.getCurrentNrparcnegocio();
			row.CDOPERADOR = this.operadorController.currentOperador.CDOPERADOR;
			row.NMOPERADOR = this.operadorController.currentOperador.NMOPERADOR;
		});

		try {
			LoadingService.show();
			await Http.post('/perfacessparc', selectedRows);
			LoadingService.hide();
			AclAlertService.showSuccess(AclMessages.SUCCESS_ADD_OPERADORES);
			dtValidateField.setDateValue('');
			addGrid.selectedRows = [];
			await addGrid.reload();
		} catch (e) {
			LoadingService.hide();
			AclDialogService.showErrorBackend(e);
		}
	}
}
