import { Loader, Metadata, Singleton } from '@zeedhi/core';
import { Form } from '@zeedhi/common';
import { Utilities } from '@/Util/Utilities';
import { OperadorService } from '@/services/OperadorService';
import { AclHttp } from '@/services/AclHttp';
import { AclAlertService } from '@/services/AclAlertService';
import AclMessages from '@/Util/AclMessages';

@Singleton
export class OperadorTabController {
	public operadorService!: OperadorService;

	constructor() {
		this.operadorService = new OperadorService();
	}

	public operadorController = Loader.getInstance('OperadorController');

	public getOperadorFormEdit(): Form {
		return Metadata.getInstance<Form>('operadorFormEdit');
	}

	public async saveEditOperador() {
		const form = this.getOperadorFormEdit();
		if (form.validate()) {
			form.value.NRINSCRICAOPARC = Utilities.getOnlyNumberInString(form.value.NRINSCRICAOPARC);
			form.value.NRPERFILACESSO = localStorage.getItem('CDPERFILSELECIONADO');
			form.value.CDTIPOINSCRICAO = this.operadorService.getInscriptionType(form.value.NRINSCRICAOPARC);
			form.value.SHOW_FULL_OPERATOR = (window.showFullOperatorWindow || false);
			form.value.IDOPERATIVO = (form.value.IDATIVO === 'S') ? 'A' : 'I';
			form.value.ADMINISTRATOR = false;

			form.value.OLD_CPF = this.operadorController.backupRow.NRINSCRICAOPARC;
			form.value.OLD_CDCLIENTE = this.operadorController.backupRow.CDCLIENTE;
			form.value.OLD_NMOPERADOR = this.operadorController.backupRow.NMOPERADOR;

			await AclHttp.$put('/operador', form.value);
			AclAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		}
	}
}
