import { Http, Metadata, Singleton } from '@zeedhi/core';
import { Form, IGridEvent, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import AclMessages from '@/Util/AclMessages';
import { Date as DateInput } from '@zeedhi/common/types/components/zd-date/date';
import { PmLibAlertService, PmLibDialogService, PmLibUtil, PmFormSave } from '@zeedhi/pm-lib';

@Singleton
export class PerfilDeAcessoController {
	private addPerfilDeAcessoModal!: Modal;

	public containerLabel!: string;

	public actionEnableAccessProfileLabel!: string;

	get currentPerfilDeAcessoName() {
		return this.currentPerfilDeAcesso.NMPERFILACESSO ?? '';
	}

	private getCurrentNrperfilacesso() {
		return this.currentPerfilDeAcesso.NRPERFILACESSO;
	}

	/** ******************************************
	 ******* Perfil de Acesso Card Section *******
	 ******************************************** */

	public async onCreatedPerfilDeAcessoCard() {
		try {
			PmLibUtil.validateNovocodigo();
			this.findAccessControlUtilization();
			this.addPerfilDeAcessoModal = await ModalService.createFromJson('components/perfil-de-acesso/modais/addPerfilDeAcesso', true);
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	private async findAccessControlUtilization() {
		const response = await Http.get('/findAccessControlUtilization');
		let idutctrlacesweb: string;

		if (response.data.IDUTCTRLACESWEB) {
			idutctrlacesweb = response.data.IDUTCTRLACESWEB;
		} else {
			idutctrlacesweb = 'N';
		}

		localStorage.setItem('idutctrlacesweb', idutctrlacesweb);
		this.changeLabelEnableAccessProfile(idutctrlacesweb);
	}

	private changeLabelEnableAccessProfile(idutctrlacesweb: string) {
		if (idutctrlacesweb === 'N') {
			this.containerLabel = 'Access Profile (Access Control Disabled)';
			this.actionEnableAccessProfileLabel = 'Enable Access Control';
		} else {
			this.containerLabel = 'Access Profile';
			this.actionEnableAccessProfileLabel = 'Disable Access Control';
		}
	}

	public destroyPerfilDeAcessoCard() {
		this.addPerfilDeAcessoModal?.destroy();
	}

	/** ******************************************
	 ******* Perfil de Acesso Grid Section *******
	 ******************************************** */

	public isVisiblePerfilDeAcessoGrid = true;

	public currentPerfilDeAcesso: any = {};

	public getInstancePerfilAcessoGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('perfilDeAcessoGrid');
	}

	public getColumnStyle(params: any) {
		return params.row.IDATIVO === 'N' ? 'acl-grid-column-inactive' : '';
	}

	public async onMountedPerfilDeAcessoGrid() {
		const grid = this.getInstancePerfilAcessoGrid();
		if (grid.datasource instanceof TekRestDatasource) {
			grid.datasource.dynamicFilter = {
				IDATIVO: [
					{
						operation: 'EQUALS',
						relation: 'AND',
						value: 'S',
					},
				],
			};

			try {
				await grid.datasource.get();
			} catch (e) {
				PmLibDialogService.showErrorBackend(e);
			}
		}
	}

	public addClickPerfilDeAcessoGrid() {
		this.addPerfilDeAcessoModal.show();
	}

	public closeAddPerfilDeAcessoModal() {
		this.addPerfilDeAcessoModal.hide();
	}

	public async saveAddPerfilDeAcessoModal() {
		const addForm = Metadata.getInstance<PmFormSave>('addPerfilDeAcessoModalForm');
		const grid = this.getInstancePerfilAcessoGrid();

		if (addForm.validate()) {
			try {
				LoadingService.show();
				await Http.post('/perfilacesso', addForm.value);
				LoadingService.hide();
				this.addPerfilDeAcessoModal.hide();
				PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
				await grid.reload();
			} catch (e) {
				LoadingService.hide();
				PmLibDialogService.showErrorBackend(e);
			}
		}
	}

	public async beforeSavePerfilDeAcessoGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const grid = this.getInstancePerfilAcessoGrid();
		const modifiedItems = grid.getEditedRows();

		try {
			LoadingService.show();
			await Http.put('/perfilacesso', modifiedItems);
			LoadingService.hide();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
			await grid.reload();
			await grid.cancelEditedRows();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	public async beforeDeletePerfilDeAcessoGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deletePerfiDeAcesso();
		}
	}

	private async deletePerfiDeAcesso() {
		try {
			const grid = this.getInstancePerfilAcessoGrid();
			LoadingService.show();
			await Http.delete('/perfilacesso', { data: { rows: grid.selectedRows } });
			LoadingService.hide();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
			await grid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	public async cellClickPerfilDeAcessoGrid({ row }: IGridEvent) {
		const operatorGrid = Metadata.getInstance<TekGrid>('perfilDeAcessoOperadorGrid');
		this.isVisiblePerfilDeAcessoGrid = false;
		this.currentPerfilDeAcesso = row;
		await operatorGrid.datasource.addFilter('NRPERFILACESSO', row.NRPERFILACESSO);
	}

	/** *******************************************
	 ******* Perfil de Acesso Detail Section *******
	 ******************************************** */

	get isVisiblePerfilDeAcessoDetail(): boolean {
		return !this.isVisiblePerfilDeAcessoGrid;
	}

	public clickBackToPerfilDeAcessoGrid() {
		this.isVisiblePerfilDeAcessoGrid = true;
	}

	/** ********************************************
	 ************ Operadores Tab Section ************
	 ********************************************* */

	/** Grid Operadores Component */
	private getPerfilDeAcessoOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('perfilDeAcessoOperadorGrid');
	}

	public async addClickPerfilDeAcessoOperadorGrid() {
		await this.createAddOperadorByPerfilModal();
		await this.addOperadorByPerfilModal.show();
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		const nrperfilacesso = this.currentPerfilDeAcesso.NRPERFILACESSO;
		await addOperadorModalGrid.datasource.addFilter('NRPERFILACESSO', nrperfilacesso);
	}

	public async beforeDeletePerfilDeAcessoOperadorGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deleteOperador();
		}
	}

	public async deleteOperador() {
		try {
			const grid = this.getPerfilDeAcessoOperadorGrid();
			LoadingService.show();
			await Http.delete('/perfacessparc', { data: grid.selectedRows });
			LoadingService.hide();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
			await grid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	/** Add Form Modal Operadores Component */

	public addOperadorByPerfilModal!: Modal;

	public async createAddOperadorByPerfilModal() {
		this.addOperadorByPerfilModal = await ModalService.createFromJson('components/perfil-de-acesso/modais/addOperadorByPerfil', true);
	}

	private getAddOperadorByPerfilGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('addOperadorByPerfilGrid');
	}

	public async closeAddOperadorByPerfilModal() {
		this.addOperadorByPerfilModal.hide();
		this.addOperadorByPerfilModal?.destroy();
		await this.getPerfilDeAcessoOperadorGrid().reload();
	}

	public get validateSaveButtonAddOperadorByPerfilModal(): boolean {
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		return !addOperadorModalGrid.selectedRows.length;
	}

	public async saveAddOperadorByPerfilModal() {
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		const addOperadorModalForm = Metadata.getInstance<Form>('addOperadorByPerfilForm');
		const dtValidateField = addOperadorModalForm.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = addOperadorModalGrid;
		selectedRows.forEach((row) => {
			row.DTVALIDADE = dtValidateField.value;
			row.NRPERFILACESSO = this.getCurrentNrperfilacesso();
		});
		try {
			LoadingService.show();
			await Http.post('/perfacessparc', selectedRows);
			LoadingService.hide();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_ADD_OPERADORES);
			dtValidateField.setDateValue('');
			addOperadorModalGrid.selectedRows = [];
			await addOperadorModalGrid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}
}
