import { Http, Loader, Metadata, Singleton } from '@zeedhi/core';
import { Form, Modal, ModalService, Date as DateInput, LoadingService, IGridEvent } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { AclDialogService } from '@/services/AclDialogService';
import { AclAlertService } from '@/services/AclAlertService';
import AclMessages from '@/Util/AclMessages';

@Singleton
export class PerfilDeAcessoOperadorController {
	public perfilDeAcessoController = Loader.getInstance('PerfilDeAcessoController');

	get isVisibleTab(): boolean {
		return !this.perfilDeAcessoController.isVisiblePerfilDeAcessoGrid;
	}

	public get currentPerfilDeAcessoName() {
		return this.perfilDeAcessoController.currentPerfilDeAcesso.NMPERFILACESSO ?? '';
	}

	private getCurrentNrperfilacesso() {
		return this.perfilDeAcessoController.currentPerfilDeAcesso.NRPERFILACESSO;
	}

	public clickBackToPerfilDeAcessoGrid() {
		this.perfilDeAcessoController.isVisiblePerfilDeAcessoGrid = true;
	}

	private getPerfilDeAcessoOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('perfilDeAcessoOperadorGrid');
	}

	private getAddOperadorByPerfilGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('addOperadorByPerfilGrid');
	}

	public async addClickPerfilDeAcessoOperadorGrid() {
		await this.createAddOperadorByPerfilModal();
		await this.addOperadorByPerfilModal.show();
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		const nrperfilacesso = this.perfilDeAcessoController.currentPerfilDeAcesso.NRPERFILACESSO;
		await addOperadorModalGrid.datasource.addFilter('NRPERFILACESSO', nrperfilacesso);
	}

	public async beforeDeletePerfilDeAcessoOperadorGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await AclDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deleteOperador();
		}
	}

	public async deleteOperador() {
		try {
			const grid = this.getPerfilDeAcessoOperadorGrid();
			LoadingService.show();
			await Http.delete('/perfacessparc', { data: grid.selectedRows });
			LoadingService.hide();
			AclAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
			await grid.reload();
		} catch (e) {
			LoadingService.hide();
			AclDialogService.showErrorBackend(e);
		}
	}

	/**
	 * Modal Add Operador
	 */

	public addOperadorByPerfilModal!: Modal;

	public async createAddOperadorByPerfilModal() {
		this.addOperadorByPerfilModal = await ModalService.createFromJson('components/perfil-de-acesso/modais/addOperadorByPerfil', true);
	}

	public async closeAddOperadorByPerfilModal() {
		this.addOperadorByPerfilModal.hide();
		this.addOperadorByPerfilModal?.destroy();
		await this.getPerfilDeAcessoOperadorGrid().reload();
	}

	public get validateSaveButtonAddOperadorByPerfilModal(): boolean {
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		return !addOperadorModalGrid.selectedRows.length;
	}

	public async saveAddOperadorByPerfilModal() {
		const addOperadorModalGrid = this.getAddOperadorByPerfilGrid();
		const addOperadorModalForm = Metadata.getInstance<Form>('addOperadorByPerfilForm');
		const dtValidateField = addOperadorModalForm.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = addOperadorModalGrid;
		selectedRows.forEach((row) => {
			row.DTVALIDADE = dtValidateField.value;
			row.NRPERFILACESSO = this.getCurrentNrperfilacesso();
		});
		try {
			LoadingService.show();
			await Http.post('/perfacessparc', selectedRows);
			LoadingService.hide();
			AclAlertService.showSuccess(AclMessages.SUCCESS_ADD_OPERADORES);
			dtValidateField.setDateValue('');
			addOperadorModalGrid.selectedRows = [];
			await addOperadorModalGrid.reload();
		} catch (e) {
			LoadingService.hide();
			AclDialogService.showErrorBackend(e);
		}
	}
}
